@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland <vadim@vk.crocodile.org>
 
-  $Id: Address.cpp 166 2008-08-03 18:18:06Z vadim $
+  $Id: Address.cpp 206 2008-10-18 20:36:18Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -101,6 +101,11 @@ bool Address::hasInetAddress() const
     return false;
 }
 
+int Address::countInetAddresses(bool ) const
+{
+    return 0;
+}
+
 const InetAddr* Address::getAddressPtr() const
 {
     const InetAddrMask *inet_addr_mask = getInetAddrMaskObjectPtr();
