@@ -6,7 +6,7 @@
 
   Author:  Illiya Yalovoy <yalovoy@gmail.com>
 
-  $Id: AddressTable.cpp 192 2008-09-21 09:39:24Z vadim $
+  $Id: AddressTable.cpp 221 2008-11-28 08:35:37Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -48,12 +48,14 @@ const char *AddressTable::TYPENAME={"AddressTable"};
 AddressTable::AddressTable() : MultiAddress() 
 {
     setRunTime(false);
+    setStr("filename", "");
 }
 
 AddressTable::AddressTable(const FWObject *root,bool prepopulate) :
     MultiAddress(root,prepopulate) 
 {
     setRunTime(false);
+    setStr("filename", "");
 }
 
 string AddressTable::getSourceName()
@@ -88,7 +90,7 @@ xmlNodePtr AddressTable::toXML(xmlNodePtr parent) throw(FWException)
 
     xmlNewProp(me, TOXMLCAST("name"), STRTOXMLCAST(getName()));
     xmlNewProp(me, TOXMLCAST("comment"), STRTOXMLCAST(getComment()));
-    xmlNewProp(me, TOXMLCAST("ro"), TOXMLCAST(((_getRO()) ? "True" : "False")));
+    xmlNewProp(me, TOXMLCAST("ro"), TOXMLCAST(((getRO()) ? "True" : "False")));
 
     return me;
 }
