@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: CustomService.cpp 192 2008-09-21 09:39:24Z vadim $
+  $Id: CustomService.cpp 202 2008-10-12 04:02:18Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -104,7 +104,7 @@ xmlNodePtr CustomService::toXML(xmlNodePtr parent) throw(FWException)
     xmlNodePtr me = FWObject::toXML(parent);
     xmlNewProp(me, TOXMLCAST("name"), STRTOXMLCAST(getName()));
     xmlNewProp(me, TOXMLCAST("comment"), STRTOXMLCAST(getComment()));
-    xmlNewProp(me, TOXMLCAST("ro"), TOXMLCAST(((_getRO()) ? "True" : "False")));
+    xmlNewProp(me, TOXMLCAST("ro"), TOXMLCAST(((getRO()) ? "True" : "False")));
 
     map<string, string>::const_iterator i;
     for(i=codes.begin(); i!=codes.end(); ++i)  
