@@ -6,7 +6,7 @@
 
   Author:  Illiya Yalovoy <yalovoy@gmail.com>
 
-  $Id: DNSName.cpp 192 2008-09-21 09:39:24Z vadim $
+  $Id: DNSName.cpp 215 2008-10-25 17:10:40Z svn $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -106,7 +106,7 @@ xmlNodePtr DNSName::toXML(xmlNodePtr parent) throw(FWException)
 
     xmlNewProp(me, TOXMLCAST("name"), STRTOXMLCAST(getName()));
     xmlNewProp(me, TOXMLCAST("comment"), STRTOXMLCAST(getComment()));
-    xmlNewProp(me, TOXMLCAST("ro"), TOXMLCAST(((_getRO()) ? "True" : "False")));
+    xmlNewProp(me, TOXMLCAST("ro"), TOXMLCAST(((getRO()) ? "True" : "False")));
 
     return me;
 }
@@ -120,10 +120,11 @@ void DNSName::loadFromSource(bool ipv6) throw(FWException)
         list<InetAddr> v = DNS::getHostByName(getSourceName(), af_type);
         for (list<InetAddr>::iterator i=v.begin(); i!=v.end(); ++i)
         {
-            Address *a = Address::cast(getRoot()->create(
-                                         (ipv6)?IPv6::TYPENAME:IPv4::TYPENAME));
+            Address *a = Address::cast(
+                getRoot()->create((ipv6)?IPv6::TYPENAME:IPv4::TYPENAME));
             a->setAddress( *i );
             addRef(a);
+
         }
     } catch (const FWException &ex)
     {
