@@ -6,7 +6,7 @@
 
   Author:  Illiya Yalovoy <yalovoy@gmail.com>
 
-  $Id: DNSName.h,v 1.6 2006/09/10 22:40:36 vkurland Exp $
+  $Id: DNSName.h 177 2008-08-23 23:48:45Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -44,7 +44,16 @@ class DNSName : public MultiAddress
     DNSName();
     DNSName(const FWObject *root,bool prepopulate);
 
-    virtual void loadFromSource() throw(FWException);
+    virtual void fromXML(xmlNodePtr parent) throw(FWException);
+    virtual xmlNodePtr toXML(xmlNodePtr xml_parent_node) throw(FWException);
+    
+    virtual std::string getSourceName();
+    virtual void setSourceName(const std::string& source_name);
+
+    std::string getDNSRecordType();
+    void setDNSRecordType(const std::string& rectype);
+    
+    virtual void loadFromSource(bool ipv6) throw(FWException);
 };
 
 }
