@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: FWOptions.h,v 1.3 2005/09/26 12:02:23 vkurland Exp $
+  $Id: FWOptions.h 35 2008-04-26 19:13:33Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -37,7 +37,7 @@ class FWOptions : public FWObject
 {
     public:
     FWOptions();
-    FWOptions(const FWObject *root);
+    FWOptions(const FWObject *root,bool prepopulate);
     DECLARE_FWOBJECT_SUBTYPE(FWOptions);
     virtual void fromXML(xmlNodePtr parent) throw(FWException);
     virtual xmlNodePtr toXML(xmlNodePtr parent) throw(FWException);
@@ -47,7 +47,7 @@ class HostOptions : public FWOptions
 {
     public:
     HostOptions();
-    HostOptions(const FWObject *root);
+    HostOptions(const FWObject *root,bool prepopulate);
     DECLARE_FWOBJECT_SUBTYPE(HostOptions);
 };
 
@@ -55,7 +55,7 @@ class FirewallOptions : public FWOptions
 {
     public:
     FirewallOptions();
-    FirewallOptions(const FWObject *root);
+    FirewallOptions(const FWObject *root,bool prepopulate);
     DECLARE_FWOBJECT_SUBTYPE(FirewallOptions);
 };
 
@@ -64,7 +64,7 @@ class PolicyRuleOptions : public FWOptions
     public:
     
     PolicyRuleOptions();
-    PolicyRuleOptions(const FWObject *root);
+    PolicyRuleOptions(const FWObject *root,bool prepopulate);
     DECLARE_FWOBJECT_SUBTYPE(PolicyRuleOptions);
 };
 
@@ -73,7 +73,7 @@ class NATRuleOptions : public FWOptions
     public:
 
     NATRuleOptions();
-    NATRuleOptions(const FWObject *root);
+    NATRuleOptions(const FWObject *root,bool prepopulate);
     DECLARE_FWOBJECT_SUBTYPE(NATRuleOptions);
 };
 
@@ -82,7 +82,7 @@ class RoutingRuleOptions : public FWOptions
     public:
 
     RoutingRuleOptions();
-    RoutingRuleOptions(const FWObject *root);
+    RoutingRuleOptions(const FWObject *root,bool prepopulate);
     DECLARE_FWOBJECT_SUBTYPE(RoutingRuleOptions);
 };
 
