@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: Logger.h,v 1.4 2007/08/02 05:19:28 vkurland Exp $
+  $Id: Logger.h 121 2008-07-02 23:00:03Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -28,38 +28,26 @@
 #ifndef __LOGGER_HH_FLAG__
 #define __LOGGER_HH_FLAG__
 
-#include <vector>
-#include <iostream>
 #include <sstream>
 
 #include <fwbuilder/FWException.h>
 #include <fwbuilder/Tools.h>
 #include <fwbuilder/ThreadTools.h>
-#include <fwbuilder/Pool.h>
-
-// #include <sigc++/signal_system.h>
 
 namespace libfwbuilder
 {
 
-class Logger;
-
-// extern Logger &start (Logger &);
-// extern Logger &end   (Logger &);
-
 class Logger
 {
     protected:
 
     Mutex line_lock;
     bool  blackhole_mode;
-
-//    friend Logger &start (Logger &);
-//    friend Logger &end   (Logger &);
+    bool  copy_to_stderr;
 
     public:
 
-    Logger() { blackhole_mode=false; }
+    Logger() { blackhole_mode=false; copy_to_stderr=false; }
     virtual ~Logger() {};
 
     virtual Logger& operator<< (char c)            = 0;
@@ -78,7 +66,8 @@ class Logger
 
     virtual bool   ready()        { return true; }
     virtual std::string getLine() { return std::string(""); }
-    void    blackhole();
+    void blackhole();
+    void copyToStderr() { copy_to_stderr=true; }
 };
 
 class NullLogger:public Logger
