@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland <vadim@fwbuilder.org>
 
-  $Id: MultiAddress.h,v 1.4 2006/09/10 22:40:37 vkurland Exp $
+  $Id: MultiAddress.h 171 2008-08-16 03:15:46Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -40,24 +40,18 @@ namespace libfwbuilder
 class MultiAddress : public ObjectGroup
 {
     private:
-    // the name of the source XML attribute
-    // (e.g. 'file' for address tables and 'dnsrec' for DNSName)
-    std::string source_attribute_name;
-
       
     public:
-    virtual void fromXML(xmlNodePtr parent) throw(FWException);
 
     DECLARE_FWOBJECT_SUBTYPE(MultiAddress);
 
     MultiAddress();
     MultiAddress(const FWObject *root,bool prepopulate);
-    void registerSourceAttributeName(const std::string &an);
+    virtual ~MultiAddress();
 
-    std::string getSourceName() const;
-    void setSourceName(const std::string &s);
-
-    virtual void loadFromSource() throw(FWException);
+    virtual std::string getSourceName();
+    virtual void setSourceName(const std::string& source_name);
+    virtual void loadFromSource(bool ipv6) throw(FWException);
 
     bool isCompileTime() const;
     bool isRunTime() const;
@@ -65,9 +59,6 @@ class MultiAddress : public ObjectGroup
     void setRunTime(const bool b); 
 
     virtual bool validateChild(FWObject *o);
-
-    virtual xmlNodePtr toXML(xmlNodePtr xml_parent_node) throw(FWException);
-
 };
 
 /*
