@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: NAT.cpp,v 1.5 2006/09/10 22:40:37 vkurland Exp $
+  $Id: NAT.cpp 109 2008-06-10 23:43:36Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -39,14 +39,12 @@ const char *NAT::TYPENAME={"NAT"};
  
 NAT::NAT() : RuleSet() 
 {
-    remStr("name");
-    remStr("comment");
+    setName("NAT");
 }
 
 NAT::NAT(const FWObject *root,bool prepopulate) : RuleSet(root,prepopulate) 
 {
-    remStr("name");
-    remStr("comment");
+    setName("NAT");
 }
 
 NAT::~NAT()  {}
