@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: Network.cpp 193 2008-09-24 13:20:49Z vadim $
+  $Id: Network.cpp 202 2008-10-12 04:02:18Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -87,7 +87,7 @@ xmlNodePtr Network::toXML(xmlNodePtr xml_parent_node) throw(FWException)
     xmlNodePtr me = FWObject::toXML(xml_parent_node);
     xmlNewProp(me, TOXMLCAST("name"), STRTOXMLCAST(getName()));
     xmlNewProp(me, TOXMLCAST("comment"), STRTOXMLCAST(getComment()));
-    xmlNewProp(me, TOXMLCAST("ro"), TOXMLCAST(((_getRO()) ? "True" : "False")));
+    xmlNewProp(me, TOXMLCAST("ro"), TOXMLCAST(((getRO()) ? "True" : "False")));
     
     xmlNewProp(me, 
                TOXMLCAST("address"),
