@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: Network.h 166 2008-08-03 18:18:06Z vadim $
+  $Id: Network.h 206 2008-10-18 20:36:18Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -54,6 +54,11 @@ public:
 
     virtual bool hasInetAddress() const { return true; }
 
+    /**
+     * similar to hasInetAddress() but counts addresses
+     */
+    virtual int countInetAddresses(bool) const { return 1; }
+    
     virtual const Address* getAddressObject() const { return this; }
 
     virtual void setAddress(const InetAddr &a);
