@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: ObjectGroup.h,v 1.2 2004/09/08 05:34:51 vkurland Exp $
+  $Id: ObjectGroup.h 177 2008-08-23 23:48:45Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -38,11 +38,13 @@ class ObjectGroup : public Group
     public:
 
     ObjectGroup();
-    ObjectGroup(const FWObject *root);
+    ObjectGroup(const FWObject *root,bool prepopulate);
     virtual ~ObjectGroup();
 
     DECLARE_FWOBJECT_SUBTYPE(ObjectGroup);
 
+    virtual xmlNodePtr toXML(xmlNodePtr xml_parent_node) throw(FWException);
+
     /*
      * verify whether given object type is approppriate as a child
      */
