@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: Resources.cpp,v 1.4 2006/04/23 18:26:39 vkurland Exp $
+  $Id: Resources.cpp 160 2008-07-21 15:46:28Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -40,7 +40,7 @@
 #include <iostream>
 #include <fstream>
 #include <cstdlib>
-
+#include <cstring>
 
 #include "fwbuilder/Resources.h"
 #include "fwbuilder/Constants.h"
@@ -220,9 +220,9 @@ bool    Resources::getResourceBool(const string& resource_path)
 }
 
 string  Resources::getObjResourceStr(const FWObject *obj,
-					  const string& resource_name)
+                                     const string& resource_name)
 {
-    string objid   = obj->getId();
+    string objid   = FWObjectDatabase::getStringId(obj->getId());
     string objtype = obj->getTypeName();
     string res;
     string resource="/FWBuilderResources/Object/"+objid+"/"+resource_name;
@@ -240,7 +240,8 @@ string  Resources::getObjResourceStr(const FWObject *obj,
 
     if (res.empty())
         cerr << "Failed to locate resource for object " << obj->getName() 
-             << " (" << obj->getId() << "), resource="
+             << " (type=" << objtype << "), "
+             << " (id=" << obj->getId() << "), resource name: "
              << resource_name << endl;
 
     return res;
