@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: Rule.cpp 177 2008-08-23 23:48:45Z vadim $
+  $Id: Rule.cpp 218 2008-11-01 19:42:59Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -84,14 +84,15 @@ void Rule::setRuleGroupName(const std::string &group_name)
 
 
 
-FWObject& Rule::shallowDuplicate(const FWObject *x, bool preserve_id) throw(FWException)
+FWObject& Rule::shallowDuplicate(const FWObject *x,
+                                 bool preserve_id) throw(FWException)
 {
     const Rule *rx=Rule::constcast(x);
-    fallback=rx->fallback;
-    hidden=rx->hidden;
-    label=rx->label;
-    unique_id=rx->unique_id;
-    abs_rule_number=rx->abs_rule_number;
+    fallback = rx->fallback;
+    hidden = rx->hidden;
+    label = rx->label;
+    unique_id = rx->unique_id;
+    abs_rule_number = rx->abs_rule_number;
 
     return  FWObject::shallowDuplicate(x,preserve_id);
 }
@@ -103,7 +104,8 @@ const char *PolicyRule::TYPENAME={"PolicyRule"};
 
 PolicyRule::PolicyRule()
 {
-    setStr("action","Deny");
+//    setStr("action","Deny");
+    setAction(PolicyRule::Deny);
 
     src_re = NULL;
     dst_re = NULL;
@@ -114,7 +116,8 @@ PolicyRule::PolicyRule()
 
 PolicyRule::PolicyRule(const FWObject *root,bool prepopulate) : Rule(root,prepopulate)
 {
-    setStr("action","Deny");
+//    setStr("action","Deny");
+    setAction(PolicyRule::Deny);
 
     src_re = NULL;
     dst_re = NULL;
@@ -128,6 +131,7 @@ PolicyRule::PolicyRule(const FWObject *root,bool prepopulate) : Rule(root,prepop
         FWObjectDatabase *db=(FWObjectDatabase*)root;
         assert(db);
 
+// <!ELEMENT PolicyRule (Src,Dst,Srv?,Itf?,When?,PolicyRuleOptions?)>
         re=db->create("Src");  assert(re!=NULL);
         add(re); src_re = RuleElementSrc::cast(re);
 
@@ -137,17 +141,18 @@ PolicyRule::PolicyRule(const FWObject *root,bool prepopulate) : Rule(root,prepop
         re=db->create("Srv");  assert(re!=NULL);
         add(re); srv_re = RuleElementSrv::cast(re);
 
-        re=db->create("When"); assert(re!=NULL);
-        add(re); when_re = RuleElementInterval::cast(re);
-
         re=db->create("Itf");  assert(re!=NULL);
         add(re); itf_re = RuleElementItf::cast(re);
 
+        re=db->create("When"); assert(re!=NULL);
+        add(re); when_re = RuleElementInterval::cast(re);
+
         add( db->create(PolicyRuleOptions::TYPENAME) );
     }
 }
 
-FWObject& PolicyRule::shallowDuplicate(const FWObject *x, bool preserve_id) throw(FWException)
+FWObject& PolicyRule::shallowDuplicate(const FWObject *x,
+                                       bool preserve_id) throw(FWException)
 {
     const PolicyRule *rx=PolicyRule::constcast(x);
     setDirection(rx->getDirection());
@@ -160,46 +165,60 @@ FWObject& PolicyRule::shallowDuplicate(const FWObject *x, bool preserve_id) thro
     itf_re = NULL;
     when_re = NULL;
 
-    return  Rule::shallowDuplicate(x,preserve_id);
+    return  Rule::shallowDuplicate(x, preserve_id);
 }
 
-
+// <!ELEMENT PolicyRule (Src,Dst,Srv?,Itf?,When?,PolicyRuleOptions?)>
 RuleElementSrc*  PolicyRule::getSrc()
 {
     if (src_re) return src_re;
-    src_re = RuleElementSrc::cast(getFirstByType(RuleElementSrc::TYPENAME));
+    FWObject::iterator i1 = begin();
+    src_re = RuleElementSrc::cast(*i1);
     return src_re;
 }
 
 RuleElementDst*  PolicyRule::getDst()
 {
     if (dst_re) return dst_re;
-    dst_re = RuleElementDst::cast(getFirstByType(RuleElementDst::TYPENAME));
+    FWObject::iterator i1 = begin();
+    i1++;
+    dst_re = RuleElementDst::cast(*i1);
     return dst_re;
 }
 
 RuleElementSrv*  PolicyRule::getSrv()
 {
     if (srv_re) return srv_re;
-    srv_re = RuleElementSrv::cast(getFirstByType(RuleElementSrv::TYPENAME));
+    FWObject::iterator i1 = begin();
+    i1++;
+    i1++;
+    srv_re = RuleElementSrv::cast(*i1);
     return srv_re;
 }
 
 RuleElementItf*  PolicyRule::getItf()
 {
     if (itf_re) return itf_re;
-    itf_re = RuleElementItf::cast(getFirstByType(RuleElementItf::TYPENAME));
+    FWObject::iterator i1 = begin();
+    i1++;
+    i1++;
+    i1++;
+    itf_re = RuleElementItf::cast(*i1);
     return itf_re;
 }
 
 RuleElementInterval* PolicyRule::getWhen()
 {
     if (when_re) return when_re;
-    when_re = RuleElementInterval::cast(getFirstByType(RuleElementInterval::TYPENAME));
+    FWObject::iterator i1 = begin();
+    i1++;
+    i1++;
+    i1++;
+    i1++;
+    when_re = RuleElementInterval::cast(*i1);
     return when_re;
 }
 
-
 bool PolicyRule::isEmpty()
 {
   return (getSrc()->isAny() && 
@@ -208,86 +227,62 @@ bool PolicyRule::isEmpty()
           getItf()->isAny());
 }
 
-PolicyRule::Action PolicyRule::getAction()  const
-{
-    string a=getActionAsString();
-    if (a=="Accept")     return Accept;
-    if (a=="Reject")     return Reject;
-    if (a=="Scrub")      return Scrub;
-    if (a=="Return")     return Return;
-    if (a=="Skip")       return Skip;
-    if (a=="Continue")   return Continue;
-    if (a=="Accounting") return Accounting;
-    if (a=="Modify")     return Modify;
-    if (a=="Tag")        return Tag;
-    if (a=="Pipe")       return Pipe;
-    if (a=="Classify")   return Classify;
-    if (a=="Custom")     return Custom;
-    if (a=="Branch")     return Branch;
-    if (a=="Route")      return Route;
-    return Deny;
-}
-
-void   PolicyRule::setAction(PolicyRule::Action act)
-{
-    switch (act) {
-    case Accept:     setAction("Accept");   return;
-    case Reject:     setAction("Reject");   return;
-    case Scrub:      setAction("Scrub");    return;
-    case Return:     setAction("Return");   return;
-    case Skip:       setAction("Skip");     return;
-    case Continue:   setAction("Continue"); return;
-    case Accounting: setAction("Accounting"); return;
-    case Modify:     setAction("Modify");   return;
-    case Tag:        setAction("Tag");      return;
-    case Pipe:       setAction("Pipe");     return;
-    case Classify:   setAction("Classify"); return;
-    case Custom:     setAction("Custom");   return;
-    case Branch:     setAction("Branch");   return;
-    case Route:      setAction("Route");    return;
-    default:         setAction("Deny");     return;
-    }
-}
-
 string PolicyRule::getActionAsString() const
 {
-    string res = getStr("action");
-    if (res.empty()) res = "Deny";
-    return res;
-}
-
-void   PolicyRule::setAction(const string& act)
-{
-    setStr("action",(act.empty())?"Deny":act);
-}
-
-PolicyRule::Direction PolicyRule::getDirection() const
-{
-    string d=getDirectionAsString();
-    if (d=="Inbound")   return Inbound;
-    if (d=="Outbound")  return Outbound;
-    return Both;
-}
-
-void   PolicyRule::setDirection(PolicyRule::Direction dir)
-{
-    switch (dir) {
-    case Inbound:   setDirection("Inbound");  return;
-    case Outbound:  setDirection("Outbound"); return;
-    default  :      setDirection("Both");     return;
+    switch (action) {
+    case Accept:     return "Accept";
+    case Reject:     return "Reject";
+    case Scrub:      return "Scrub";
+    case Return:     return "Return";
+    case Skip:       return "Skip";
+    case Continue:   return "Continue";
+    case Accounting: return "Accounting";
+    case Modify:     return "Modify";
+    case Tag:        return "Tag";
+    case Pipe:       return "Pipe";
+    case Classify:   return "Classify";
+    case Custom:     return "Custom";
+    case Branch:     return "Branch";
+    case Route:      return "Route";
     }
+    return "Deny";
+}
+
+void PolicyRule::setAction(const string& act)
+{
+    if (act=="Accept")     { setAction(Accept); return; }
+    if (act=="Deny")       { setAction(Deny); return; }
+    if (act=="Reject")     { setAction(Reject); return; }
+    if (act=="Scrub")      { setAction(Scrub); return; }
+    if (act=="Return")     { setAction(Return); return; }
+    if (act=="Skip")       { setAction(Skip); return; }
+    if (act=="Continue")   { setAction(Continue); return; }
+    if (act=="Accounting") { setAction(Accounting); return; }
+    if (act=="Modify")     { setAction(Modify); return; }
+    if (act=="Tag")        { setAction(Tag); return; }
+    if (act=="Pipe")       { setAction(Pipe); return; }
+    if (act=="Classify")   { setAction(Classify); return; }
+    if (act=="Custom")     { setAction(Custom); return; }
+    if (act=="Branch")     { setAction(Branch); return; }
+    if (act=="Route")      { setAction(Route); return; }
+    setAction(Deny);
 }
 
 string PolicyRule::getDirectionAsString() const
 {
-    string res = getStr("direction");
-    if (res.empty()) res = "Both";
-    return res;
+    switch (direction)
+    {
+    case Inbound:   return "Inbound";
+    case Outbound:  return "Outbound";
+    }
+    return "Both";
 }
 
-void   PolicyRule::setDirection(const string& dir)
+void PolicyRule::setDirection(const string& dir)
 {
-    setStr("direction",(dir.empty())?"Both":dir);
+    if (dir=="Inbound")   { setDirection(Inbound); return; }
+    if (dir=="Outbound")  { setDirection(Outbound); return; }
+    setDirection(Both);
 }
 
 bool   PolicyRule::getLogging() const    { return getBool("log"); }
@@ -317,7 +312,7 @@ void PolicyRule::fromXML(xmlNodePtr root) throw(FWException)
     n=FROMXMLCAST(xmlGetProp(root,TOXMLCAST("action")));
     if(n)
     {
-        setStr("action",n);
+        setAction(string(n));
         FREEXMLBUFF(n);
     }
 
@@ -339,7 +334,7 @@ void PolicyRule::fromXML(xmlNodePtr root) throw(FWException)
     n= FROMXMLCAST(xmlGetProp(root,TOXMLCAST("direction")));
     if(n)
     {
-        setStr("direction",n);
+        setDirection(string(n));
         FREEXMLBUFF(n);
     }
 
@@ -355,11 +350,19 @@ void PolicyRule::fromXML(xmlNodePtr root) throw(FWException)
 xmlNodePtr PolicyRule::toXML(xmlNodePtr parent) throw(FWException)
 {
     xmlNodePtr me = FWObject::toXML(parent, false);
-//    xmlNewProp(me, TOXMLCAST("name"), STRTOXMLCAST(getName()));
+    xmlNewProp(me, TOXMLCAST("action"), STRTOXMLCAST(getActionAsString()));
+    xmlNewProp(me, TOXMLCAST("direction"),STRTOXMLCAST(getDirectionAsString()));
     xmlNewProp(me, TOXMLCAST("comment"), STRTOXMLCAST(getComment()));
 
     FWObject *o;
-    
+
+    /*
+     * Save children to XML file in just this order (src, dst, srv).
+     * PolicyCompiler::checkForShadowing depends on it!
+     * But after all, DTD requires this order.
+     *
+     <!ELEMENT PolicyRule (Src,Dst,Srv?,Itf?,When?,PolicyRuleOptions?)>
+     */
     if ( (o=getFirstByType( RuleElementSrc::TYPENAME ))!=NULL )
 	o->toXML(me);
 
@@ -378,6 +381,7 @@ xmlNodePtr PolicyRule::toXML(xmlNodePtr parent) throw(FWException)
     if ( (o=getFirstByType( PolicyRuleOptions::TYPENAME ))!=NULL )
 	o->toXML(me);
 
+    // there should be no children Policy objects in v3
     if ( (o=getFirstByType( Policy::TYPENAME ))!=NULL )
 	o->toXML(me);
 
@@ -716,7 +720,8 @@ void         NATRule::setRuleType(NATRuleTypes rt)
     rule_type=rt;
 }
 
-FWObject& NATRule::shallowDuplicate(const FWObject *x, bool preserve_id) throw(FWException)
+FWObject& NATRule::shallowDuplicate(const FWObject *x,
+                                    bool preserve_id) throw(FWException)
 {
     const NATRule *rx=NATRule::constcast(x);
     if (rx!=NULL) rule_type=rx->rule_type;
@@ -729,7 +734,7 @@ FWObject& NATRule::shallowDuplicate(const FWObject *x, bool preserve_id) throw(F
     tsrv_re = NULL;
     when_re = NULL;
 
-    return  Rule::shallowDuplicate(x,preserve_id);
+    return  Rule::shallowDuplicate(x, preserve_id);
 }
 
 
@@ -917,14 +922,15 @@ void RoutingRule::setRuleType(RoutingRuleTypes rt)
     rule_type=rt;
 }
 
-FWObject& RoutingRule::duplicate(const FWObject *x, bool preserve_id) throw(FWException)
+FWObject& RoutingRule::duplicate(const FWObject *x,
+                                 bool preserve_id) throw(FWException)
 {
     Rule::duplicate(x,preserve_id);
-    const RoutingRule *rx=RoutingRule::constcast(x);
-    if (rx!=NULL) {
-        
-        rule_type=rx->rule_type;
-        sorted_dst_ids=rx->sorted_dst_ids;
+    const RoutingRule *rx = RoutingRule::constcast(x);
+    if (rx!=NULL)
+    {
+        rule_type = rx->rule_type;
+        sorted_dst_ids = rx->sorted_dst_ids;
     }
     return *this;
 }
