@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: RuleElement.cpp 192 2008-09-21 09:39:24Z vadim $
+  $Id: RuleElement.cpp 212 2008-10-22 04:45:16Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -53,13 +53,13 @@ const char *RuleElement::TYPENAME={"RuleElement"};
 
 RuleElement::RuleElement() 
 {
-    setBool("neg",false);
+    setNeg(false);
 }
 
 RuleElement::RuleElement(const FWObject *root, bool prepopulate) :
     FWObject(root,prepopulate)
 {
-    setBool("neg", false);
+    setNeg(false);
     setId(-1);
 }
 
@@ -68,19 +68,36 @@ void RuleElement::fromXML(xmlNodePtr root) throw(FWException)
     const char *n;
 
     n=FROMXMLCAST(xmlGetProp(root,TOXMLCAST("neg")));
-    if(n)
+    if (n)
     {
-        setStr("neg", n);
+        negation = (cxx_strcasecmp(n, "1")==0 || cxx_strcasecmp(n , "true")==0);
         FREEXMLBUFF(n);
     }
 
     FWObject::fromXML(root);
 }
 
+xmlNodePtr RuleElement::toXML(xmlNodePtr xml_parent_node) throw(FWException)
+{
+    xmlNodePtr me = FWObject::toXML(xml_parent_node);
+    xmlNewProp(me, TOXMLCAST("neg"),
+               TOXMLCAST(((getNeg()) ? "True" : "False")));
+    return me;
+}
+
+FWObject& RuleElement::shallowDuplicate(const FWObject *other,
+                                        bool preserve_id) throw(FWException)
+{
+    setNeg(RuleElement::constcast(other)->getNeg());
+    return FWObject::shallowDuplicate(other, preserve_id);
+}
+
+
 void RuleElement::addRef(FWObject *obj)
 {
     FWObject *o=NULL;
-    if (isAny()) {
+    if (isAny())
+    {
 	o=(*(begin()));
 	o=(FWReference::cast(o))->getPointer();
     }
@@ -163,28 +180,6 @@ int RuleElement::getAnyElementId()
     return -1;
 }
 
-/*
- *   Meaning of "source","destination" and "service" can be negated
- */
-bool RuleElement::getNeg()
-{
-    return ( getBool("neg") );
-}
-
-void RuleElement::setNeg(bool f) 
-{ 
-    if (!empty()) setBool("neg",f);
-    else          setBool("neg",false);  // it does not make sence to negate nothing
-}
-
-void RuleElement::toggleNeg()
-{ 
-    bool n;
-    n=getBool("neg");
-    if (!empty()) setBool("neg",!n );
-    else          setBool("neg",false);  // it does not make sence to negate nothing
-}
-
 const char *RuleElementSrc::TYPENAME={"Src"};
 RuleElementSrc::RuleElementSrc() {}
 RuleElementSrc::RuleElementSrc(const FWObject *root,bool prepopulate) :
@@ -201,6 +196,8 @@ int RuleElementSrc::getAnyElementId() {
 xmlNodePtr RuleElementSrc::toXML(xmlNodePtr parent) throw(FWException)
 {
     xmlNodePtr me = FWObject::toXML(parent, false);
+    xmlNewProp(me, TOXMLCAST("neg"),
+               TOXMLCAST(((getNeg()) ? "True" : "False")));
 
     for(list<FWObject*>::const_iterator j=begin(); j!=end(); ++j) 
         (*j)->toXML(me);
@@ -226,6 +223,8 @@ int RuleElementDst::getAnyElementId() {
 xmlNodePtr RuleElementDst::toXML(xmlNodePtr parent) throw(FWException)
 {
     xmlNodePtr me = FWObject::toXML(parent, false);
+    xmlNewProp(me, TOXMLCAST("neg"),
+               TOXMLCAST(((getNeg()) ? "True" : "False")));
 
     for(list<FWObject*>::const_iterator j=begin(); j!=end(); ++j) 
         (*j)->toXML(me);
@@ -252,6 +251,8 @@ int RuleElementSrv::getAnyElementId() {
 xmlNodePtr RuleElementSrv::toXML(xmlNodePtr parent) throw(FWException)
 {
     xmlNodePtr me = FWObject::toXML(parent, false);
+    xmlNewProp(me, TOXMLCAST("neg"),
+               TOXMLCAST(((getNeg()) ? "True" : "False")));
 
     for(list<FWObject*>::const_iterator j=begin(); j!=end(); ++j) 
         (*j)->toXML(me);
@@ -277,6 +278,8 @@ int RuleElementItf::getAnyElementId()
 xmlNodePtr RuleElementItf::toXML(xmlNodePtr parent) throw(FWException)
 {
     xmlNodePtr me = FWObject::toXML(parent, false);
+    xmlNewProp(me, TOXMLCAST("neg"),
+               TOXMLCAST(((getNeg()) ? "True" : "False")));
 
     for(list<FWObject*>::const_iterator j=begin(); j!=end(); ++j) 
         (*j)->toXML(me);
@@ -362,6 +365,8 @@ int RuleElementOSrc::getAnyElementId() {
 xmlNodePtr RuleElementOSrc::toXML(xmlNodePtr parent) throw(FWException)
 {
     xmlNodePtr me = FWObject::toXML(parent, false);
+    xmlNewProp(me, TOXMLCAST("neg"),
+               TOXMLCAST(((getNeg()) ? "True" : "False")));
 
     for(list<FWObject*>::const_iterator j=begin(); j!=end(); ++j) 
         (*j)->toXML(me);
@@ -387,6 +392,8 @@ int RuleElementODst::getAnyElementId() {
 xmlNodePtr RuleElementODst::toXML(xmlNodePtr parent) throw(FWException)
 {
     xmlNodePtr me = FWObject::toXML(parent, false);
+    xmlNewProp(me, TOXMLCAST("neg"),
+               TOXMLCAST(((getNeg()) ? "True" : "False")));
 
     for(list<FWObject*>::const_iterator j=begin(); j!=end(); ++j) 
         (*j)->toXML(me);
@@ -412,6 +419,8 @@ int RuleElementOSrv::getAnyElementId() {
 xmlNodePtr RuleElementOSrv::toXML(xmlNodePtr parent) throw(FWException)
 {
     xmlNodePtr me = FWObject::toXML(parent, false);
+    xmlNewProp(me, TOXMLCAST("neg"),
+               TOXMLCAST(((getNeg()) ? "True" : "False")));
 
     for(list<FWObject*>::const_iterator j=begin(); j!=end(); ++j) 
         (*j)->toXML(me);
@@ -439,6 +448,8 @@ int RuleElementTSrc::getAnyElementId() {
 xmlNodePtr RuleElementTSrc::toXML(xmlNodePtr parent) throw(FWException)
 {
     xmlNodePtr me = FWObject::toXML(parent, false);
+    xmlNewProp(me, TOXMLCAST("neg"),
+               TOXMLCAST(((getNeg()) ? "True" : "False")));
 
     for(list<FWObject*>::const_iterator j=begin(); j!=end(); ++j) 
         (*j)->toXML(me);
@@ -464,6 +475,8 @@ int RuleElementTDst::getAnyElementId() {
 xmlNodePtr RuleElementTDst::toXML(xmlNodePtr parent) throw(FWException)
 {
     xmlNodePtr me = FWObject::toXML(parent, false);
+    xmlNewProp(me, TOXMLCAST("neg"),
+               TOXMLCAST(((getNeg()) ? "True" : "False")));
 
     for(list<FWObject*>::const_iterator j=begin(); j!=end(); ++j) 
         (*j)->toXML(me);
@@ -489,6 +502,8 @@ int RuleElementTSrv::getAnyElementId() {
 xmlNodePtr RuleElementTSrv::toXML(xmlNodePtr parent) throw(FWException)
 {
     xmlNodePtr me = FWObject::toXML(parent, false);
+    xmlNewProp(me, TOXMLCAST("neg"),
+               TOXMLCAST(((getNeg()) ? "True" : "False")));
 
     for(list<FWObject*>::const_iterator j=begin(); j!=end(); ++j) 
         (*j)->toXML(me);
@@ -516,6 +531,8 @@ int RuleElementInterval::getAnyElementId() {
 xmlNodePtr RuleElementInterval::toXML(xmlNodePtr parent) throw(FWException)
 {
     xmlNodePtr me = FWObject::toXML(parent, false);
+    xmlNewProp(me, TOXMLCAST("neg"),
+               TOXMLCAST(((getNeg()) ? "True" : "False")));
 
     for(list<FWObject*>::const_iterator j=begin(); j!=end(); ++j) 
         (*j)->toXML(me);
@@ -542,6 +559,8 @@ int RuleElementRDst::getAnyElementId() {
 xmlNodePtr RuleElementRDst::toXML(xmlNodePtr parent) throw(FWException)
 {
     xmlNodePtr me = FWObject::toXML(parent, false);
+    xmlNewProp(me, TOXMLCAST("neg"),
+               TOXMLCAST(((getNeg()) ? "True" : "False")));
 
     for(list<FWObject*>::const_iterator j=begin(); j!=end(); ++j) 
         (*j)->toXML(me);
@@ -566,6 +585,8 @@ int RuleElementRGtw::getAnyElementId() {
 xmlNodePtr RuleElementRGtw::toXML(xmlNodePtr parent) throw(FWException)
 {
     xmlNodePtr me = FWObject::toXML(parent, false);
+    xmlNewProp(me, TOXMLCAST("neg"),
+               TOXMLCAST(((getNeg()) ? "True" : "False")));
 
     for(list<FWObject*>::const_iterator j=begin(); j!=end(); ++j) 
         (*j)->toXML(me);
