@@ -6,7 +6,7 @@
 
   Author:  Vadim Zaliva <lord@crocodile.org>
 
-  $Id: Service.cpp,v 1.5 2006/09/10 22:40:37 vkurland Exp $
+  $Id: Service.cpp 105 2008-06-09 01:52:12Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -57,5 +57,5 @@ FWReference* Service::createRef()
 
 bool Service::isAny() const
 { 
-    return getId()==FWObjectDatabase::getAnyIPServiceId();
+    return getId()==FWObjectDatabase::ANY_SERVICE_ID;
 }
