@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: TCPService.h,v 1.4 2007/05/08 01:26:21 vkurland Exp $
+  $Id: TCPService.h 105 2008-06-09 01:52:12Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -28,12 +28,12 @@
 #ifndef __TCPSERVICE_HH_FLAG__
 #define __TCPSERVICE_HH_FLAG__
 
-#include <fwbuilder/Service.h>
+#include <fwbuilder/TCPUDPService.h>
 
 namespace libfwbuilder
 {
 
-class TCPService : public Service
+class TCPService : public TCPUDPService
 {
 
     public:
@@ -43,7 +43,7 @@ class TCPService : public Service
 private:
     static std::map<TCPFlag,std::string> flags;
     static std::map<TCPFlag,std::string> flags_masks;
-
+    
     void init();
 
 public:
@@ -53,6 +53,7 @@ public:
     virtual ~TCPService();
 
     virtual void fromXML(xmlNodePtr parent) throw(FWException);
+    virtual xmlNodePtr toXML(xmlNodePtr xml_parent_node) throw(FWException);
 
     DECLARE_FWOBJECT_SUBTYPE(TCPService);
 
@@ -82,6 +83,7 @@ public:
 
     bool    getEstablished();
     void    setEstablished(bool f);
+    
 };
 
 }
