@@ -7,7 +7,7 @@
 
   Author:  Illiya Yalovoy <yalovoy@gmail.com>
 
-  $Id: TagService.h,v 1.3 2006/09/10 22:40:37 vkurland Exp $
+  $Id: TagService.h 177 2008-08-23 23:48:45Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -48,8 +48,8 @@ class TagService : public Service
 
     public:
 
-        std::string getCode();
-        void setCode(std::string p);
+    std::string getCode() const;
+    void setCode(const std::string &p);
 
     DECLARE_FWOBJECT_SUBTYPE(TagService);
 
@@ -59,6 +59,7 @@ class TagService : public Service
     
     
     virtual void fromXML(xmlNodePtr parent) throw(FWException);
+    virtual xmlNodePtr toXML(xmlNodePtr xml_parent_node) throw(FWException);
     
     virtual std::string getProtocolName();
     virtual int    getProtocolNumber();
