@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: Tools.cpp,v 1.9 2007/08/26 03:06:36 vkurland Exp $
+  $Id: Tools.cpp 170 2008-08-13 06:27:20Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -27,11 +27,13 @@
 #include <fwbuilder/libfwbuilder-config.h>
 #include <fwbuilder/Tools.h>
 #include <fwbuilder/XMLTools.h>
-#include <fwbuilder/dns.h>
 
 #include <string.h>
 #include <stdio.h>
 
+#include <iostream>
+#include <sstream>
+
 /*
  * this is workaround, by some reason dirent.h defines DIR as a typedef
  * for struct __dirstream which is not defined anywhere.  07/12/02  vk
@@ -46,7 +48,7 @@ struct __dirstream {};
 #else
 #  include <basetsd.h>
 #  include <io.h>
-#  include <windows.h>
+//#  include <windows.h>
 #  include <winsock2.h>
 #endif
 
@@ -73,13 +75,6 @@ char *cxx_strdup(const char *x)
     return res;
 }
 
-string int2string(int n)
-{
-    char x[32];
-    sprintf(x,"%d", n);
-    return x;
-}
-
 string substituteMacros(const string &source, const map<string, string> &macros, bool strict) throw(libfwbuilder::FWException)
 {
     string name;
@@ -114,7 +109,11 @@ string substituteMacros(const string &source, const map<string, string> &macros,
                     m    = MODE_NAME;
                     name = "";
                 } else
-                    throw FWException(string("Unexpected symbol after '$' at postion ")+int2string(i));
+                {
+                    ostringstream err;
+                    err << "Unexpected symbol after '$' at postion " << i;
+                    throw FWException(err.str());
+                }
             }
             break;
         case MODE_NAME:
@@ -140,7 +139,7 @@ string substituteMacros(const string &source, const map<string, string> &macros,
 
 char *cxx_strtok_r(char *s, const char *delim, char **save_ptr)
 {
-#ifndef HAVE_STRTOK_R
+#if !defined(HAVE_STRTOK_R) && !defined(HAVE_STRTOK_S)
     char *token;
 
     token = 0;     /* Initialize to no token. */
@@ -165,9 +164,13 @@ char *cxx_strtok_r(char *s, const char *delim, char **save_ptr)
     
     return token;
 #else
+#ifdef HAVE_STRTOK_S
+    return strtok_s(s, delim, save_ptr);
+#else
 //    return ::strtok_r(s, delim, save_ptr);
     return strtok_r(s, delim, save_ptr);
 #endif
+#endif
 }
 
 int  cxx_strcasecmp(const char *s1, const char *s2)
@@ -208,9 +211,6 @@ void init()
     }
 #endif
 
-    /* Initialize classes which need static initialization */
-    DNS::init();
-
     /* Init XML parser */
     XMLTools::initXMLTools();
    
