@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: UDPService.h,v 1.3 2006/09/10 22:40:37 vkurland Exp $
+  $Id: UDPService.h 105 2008-06-09 01:52:12Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -28,28 +28,24 @@
 #ifndef __UDPSERVICE_HH_FLAG__
 #define __UDPSERVICE_HH_FLAG__
 
-#include <fwbuilder/Service.h>
+#include <fwbuilder/TCPUDPService.h>
 
 namespace libfwbuilder
 {
 
-class UDPService : public Service
+class UDPService : public TCPUDPService
 {
-    
-    private:
-    
     public:
 
     UDPService();
     UDPService(const FWObject *root,bool prepopulate);
     virtual ~UDPService();
     
-    virtual void fromXML(xmlNodePtr parent) throw(FWException);
-    
     DECLARE_FWOBJECT_SUBTYPE(UDPService);
     
     virtual std::string getProtocolName();
     virtual int    getProtocolNumber();
+    
 };
     
 }
