@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland <vadim@vk.crocodile.org>
 
-  $Id: physAddress.cpp 192 2008-09-21 09:39:24Z vadim $
+  $Id: physAddress.cpp 202 2008-10-12 04:02:18Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -61,7 +61,7 @@ xmlNodePtr physAddress::toXML(xmlNodePtr parent) throw(FWException)
     xmlNodePtr me = FWObject::toXML(parent, false);
     xmlNewProp(me, TOXMLCAST("name"), STRTOXMLCAST(getName()));
     xmlNewProp(me, TOXMLCAST("comment"), STRTOXMLCAST(getComment()));
-    xmlNewProp(me, TOXMLCAST("ro"), TOXMLCAST(((_getRO()) ? "True" : "False")));
+    xmlNewProp(me, TOXMLCAST("ro"), TOXMLCAST(((getRO()) ? "True" : "False")));
 
     return me;
 }
