@@ -6,7 +6,7 @@
 
   Author:  Vadim Zaliva lord@crocodile.org
 
-  $Id: snmp.cpp,v 1.11 2007/08/02 05:19:28 vkurland Exp $
+  $Id: snmp.cpp 121 2008-07-02 23:00:03Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -155,7 +155,7 @@ const int SNMPCrawler::PTP_INTERFACE_TYPES[]={
  * temporary database object, needs this to be able to create
  * interfaces and then add ip addresses to them
  */
-static FWObjectDatabase snmp_tmp_db;
+static FWObjectDatabase *snmp_tmp_db = NULL;
 
 
 #include <iostream>
@@ -185,6 +185,10 @@ void SNMPQuery::init(string h, string c, int retries_, long timeout_)
 
     char *snmp_out_opt = (char*)("n");
     snmp_out_toggle_options(snmp_out_opt);
+
+    if (snmp_tmp_db==NULL)
+        snmp_tmp_db = new FWObjectDatabase();
+
 }
 
 SNMPQuery::~SNMPQuery()
@@ -269,7 +273,7 @@ void SNMPQuery::fetchArpTable(Logger *logger,SyncFlag *stop_program, SNMPConnect
                     SNMP_AT_TABLE_NET << "' table. Skipping it.\n";
                     continue;
                 }
-                IPAddress ip = dynamic_cast<SNMPVariable_IPaddr*>((*j).second)->getIPAddressValue();
+                InetAddr ip = dynamic_cast<SNMPVariable_IPaddr*>((*j).second)->getInetAddrValue();
 
                 str << "Learned: " << ip.toString();
 
@@ -371,7 +375,7 @@ void SNMPQuery::fetchRoutingTable(Logger *logger,SyncFlag *stop_program, SNMPCon
                     
                     continue;
                 }
-                IPAddress dst = dynamic_cast<SNMPVariable_IPaddr*>((*j).second)->getIPAddressValue();
+                InetAddr dst = dynamic_cast<SNMPVariable_IPaddr*>((*j).second)->getInetAddrValue();
                 string rname=(*j).first.substr(strlen(SNMP_ROUTE_DST_TABLE)+1);
 
                 v=c->get(string(SNMP_ROUTE_NM_TABLE)+"."+rname);
@@ -384,7 +388,7 @@ void SNMPQuery::fetchRoutingTable(Logger *logger,SyncFlag *stop_program, SNMPCon
                     SNMPVariable::freeVarList(v);
                     continue;
                 }
-                Netmask nm = dynamic_cast<SNMPVariable_IPaddr*>(v[0])->getNetmaskValue();
+                InetAddr nm = dynamic_cast<SNMPVariable_IPaddr*>(v[0])->getNetmaskValue();
                 SNMPVariable::freeVarList(v);
 
                 v=c->get(string(SNMP_ROUTE_TYPE_TABLE)+"."+rname);
@@ -401,16 +405,16 @@ void SNMPQuery::fetchRoutingTable(Logger *logger,SyncFlag *stop_program, SNMPCon
                     SNMPVariable::freeVarList(v);
                     continue;
                 }
-                IPAddress gw = dynamic_cast<SNMPVariable_IPaddr*>(v[0])->getIPAddressValue();
+                InetAddr gw = dynamic_cast<SNMPVariable_IPaddr*>(v[0])->getInetAddrValue();
                 SNMPVariable::freeVarList(v);
 
                 v=c->get(string(SNMP_ROUTE_IF_TABLE)+"."+rname);
                 int ifindex = SNMPVariable::varList2Int(v);
                 SNMPVariable::freeVarList(v);
  
-                map<int,Interface>::iterator ici=interfaces.find(ifindex);
+                map<int,InterfaceData>::iterator ici = interfaces.find(ifindex);
                 
-                Interface *route_intf;
+                InterfaceData route_intf;
                 if(ici==interfaces.end())
                 {
                     str << "Route references non-existing interface '"
@@ -418,8 +422,7 @@ void SNMPQuery::fetchRoutingTable(Logger *logger,SyncFlag *stop_program, SNMPCon
                         << string(SNMP_ROUTE_IF_TABLE)+"."+rname
                         << "' variable. Ignoring interface reference.\n";
                     *logger << str;
-                    
-                    route_intf=NULL;
+                    route_intf.name = "(interface not found)";
                 } else
                 {
 #ifdef SNMP_CRAWLER_DEBUG
@@ -429,13 +432,12 @@ void SNMPQuery::fetchRoutingTable(Logger *logger,SyncFlag *stop_program, SNMPCon
                     *logger << str;
                     
 #endif
-                    
-                    route_intf=&(*ici).second;
+                    route_intf = ici->second;
                 }
                 bool isdef= type!=SNMP_DIRECT_ROUTE &&
-                    nm.getLength() == 0 && dst == IPAddress("0.0.0.0");
+                    nm.getLength() == 0 && dst == InetAddr("0.0.0.0");
 
-		if (isdef && route_intf) route_intf->setExt(true);
+		if (isdef) route_intf.ext = true;
 
                 IPRoute rt(dst, nm, gw, route_intf, type==SNMP_DIRECT_ROUTE);
                 routes.push_back(rt);
@@ -446,8 +448,8 @@ void SNMPQuery::fetchRoutingTable(Logger *logger,SyncFlag *stop_program, SNMPCon
                     << "  gw " 
                     << rt.getGateway().toString()
                     << "  "
-                    << string( (route_intf)?rt.getInterface()->getName():"(interface not found)")
-                    << string( (route_intf && rt.getInterface()->isExt())?"(Ext)":"")
+                    << rt.getInterface().name
+                    << string( (rt.getInterface().ext)?"(Ext)":"")
                     << "\n";
                 *logger << str;
                 
@@ -480,10 +482,11 @@ bool SNMPQuery::isDefault(const IPRoute &r) const
 {
     return !r.isDirect() &&
     r.getNetmask().getLength() == 0 &&
-    r.getDestination() == IPAddress("0.0.0.0");
+    r.getDestination() == InetAddr("0.0.0.0");
 }
 
-void SNMPQuery::fetchInterfaces(Logger *logger,SyncFlag *stop_program, SNMPConnection *connection) throw(FWException)
+void SNMPQuery::fetchInterfaces(Logger *logger,SyncFlag *stop_program,
+                                SNMPConnection *connection) throw(FWException)
 {
     std::ostringstream str;
 
@@ -643,16 +646,15 @@ void SNMPQuery::fetchInterfaces(Logger *logger,SyncFlag *stop_program, SNMPConne
             int itype = SNMPVariable::varList2Int(v);
             SNMPVariable::freeVarList(v);
 
-            interfaces[ifindex]=Interface();
-            interfaces[ifindex].setName(descr);
-            interfaces[ifindex].setOStatus(ostatus);
-            interfaces[ifindex].setInterfaceType(itype);
-            interfaces[ifindex].setExt(false);
+            interfaces[ifindex] = InterfaceData();
+            interfaces[ifindex].name = descr;
+            interfaces[ifindex].snmp_type = itype;
+            interfaces[ifindex].ext = false;
+            interfaces[ifindex].ostatus = ostatus;
 
-            snmp_tmp_db.add( &interfaces[ifindex] );
+            //snmp_tmp_db->add( &interfaces[ifindex] );
 
-            if (physa!="")
-                interfaces[ifindex].setPhysicalAddress(physa);
+            if (physa!="") interfaces[ifindex].mac_addr = physa;
 
             for (list<string>::iterator ali=addlist.begin(); ali!=addlist.end(); ali++)
             {
@@ -660,7 +662,7 @@ void SNMPQuery::fetchInterfaces(Logger *logger,SyncFlag *stop_program, SNMPConne
 
                 // get netmasks
                 CHECK_STOP_AND_THROW_EXCEPTION
-                v=c->get(string(SNMP_NMASK_TABLE)+"."+aprefix);
+                v = c->get(string(SNMP_NMASK_TABLE)+"."+aprefix);
                 if(v.size()!=1)
                     throw FWException("Can't get netmask");
                 if(v[0]->type!=SNMPVariable::snmp_ipaddr)
@@ -669,23 +671,22 @@ void SNMPQuery::fetchInterfaces(Logger *logger,SyncFlag *stop_program, SNMPConne
                 SNMPVariable::freeVarList(v);
             
                 // get addresss
-                v=c->get(string(SNMP_ADDR_TABLE)+"."+aprefix);
+                v = c->get(string(SNMP_ADDR_TABLE)+"."+aprefix);
                 if(v.size()!=1)
                     throw FWException("Can't get IP address");
                 if(v[0]->type!=SNMPVariable::snmp_ipaddr)
                     throw FWException("Wrong return type for IP address");
-                string ad = dynamic_cast<SNMPVariable_IPaddr*>(v[0])->getIPAddressValue().toString();
+                string ad = dynamic_cast<SNMPVariable_IPaddr*>(v[0])->getInetAddrValue().toString();
                 SNMPVariable::freeVarList(v);
 
-                v=c->get(string(SNMP_BCAST_TABLE)+"."+aprefix);
+                v = c->get(string(SNMP_BCAST_TABLE)+"."+aprefix);
                 if(v.size()!=1)
                     throw FWException("Can't get broadcast address");
 //                long bcast_bits = SNMPVariable::varList2Int(v);
                 SNMPVariable::freeVarList(v);
 
-                IPv4 *ipaddr=interfaces[ifindex].addIPv4();
-                ipaddr->setAddress(ad);
-                ipaddr->setNetmask(nm);
+                interfaces[ifindex].addr_mask.setAddress(InetAddr(ad));
+                interfaces[ifindex].addr_mask.setNetmask(InetAddr(nm));
 
                 str << "Adding interface #" << ifindex 
                     << ": " << ad << "/" << nm
@@ -799,12 +800,12 @@ const vector<IPRoute> &SNMPQuery::getRoutes()
     return routes;
 }
 
-const map<IPAddress, string> &SNMPQuery::getArtpTable()
+const map<InetAddr, string> &SNMPQuery::getArtpTable()
 {
     return arptable;
 }
 
-const map<int, Interface> &SNMPQuery::getInterfaces()
+const map<int, InterfaceData> &SNMPQuery::getInterfaces()
 {
     return interfaces;
 }
@@ -1081,7 +1082,7 @@ string SNMPVariable_Bits::toString()
 
 string SNMPVariable_IPaddr::toString()
 {
-    string res="SNMP IPAddress/Netmask[";
+    string res="SNMP InetAddr/Netmask[";
     for(size_t i=0;i<len;i++)
     {
         if(i) 
@@ -1094,14 +1095,27 @@ string SNMPVariable_IPaddr::toString()
     return res;
 }
 
-IPAddress SNMPVariable_IPaddr::getIPAddressValue() throw(FWException)
+InetAddr SNMPVariable_IPaddr::getInetAddrValue() throw(FWException)
 {
-    return IPAddress(value, len);
+    // value comes from ASN1-encoded snmp reply. I am not sure 100% but
+    // I'll assume it consists of 4 bytes that represent ip address in
+    // network order.
+    union {
+        struct in_addr ipaddr;
+        char addr_bytes[4];
+    } addr_conversion;
+    memcpy((void*)(&addr_conversion), value, len);
+    return InetAddr(&addr_conversion.ipaddr);
 }
 
-Netmask SNMPVariable_IPaddr::getNetmaskValue() throw(FWException)
+InetAddr SNMPVariable_IPaddr::getNetmaskValue() throw(FWException)
 {
-    return Netmask(value, len);
+    union {
+        struct in_addr ipaddr;
+        char addr_bytes[4];
+    } addr_conversion;
+    memcpy((void*)(&addr_conversion), value, len);
+    return InetAddr(&addr_conversion.ipaddr);
 }
 
 string SNMPVariable_String::toString()
@@ -1175,7 +1189,7 @@ long SNMPVariable::varList2Int(vector<SNMPVariable*> &v) throw(FWException)
 SNMPCrawler::SNMPCrawler()
 {}
 
-SNMPCrawler::SNMPCrawler(const IPAddress &_seed, 
+SNMPCrawler::SNMPCrawler(const InetAddr &_seed, 
                          const string &_community,
                          bool _recursive,
                          bool _skip_virtual,
@@ -1186,7 +1200,7 @@ SNMPCrawler::SNMPCrawler(const IPAddress &_seed,
                          long _snmp_timeout,
 			 int  _dns_retries,
 			 int  _dns_timeout,
-                         const vector<IPNetwork> *_include)
+                         const vector<InetAddrMask> *_include)
 {
     init(_seed, _community, _recursive, _skip_virtual, 
          _do_dns, _follow_ptp, _dns_threads, 
@@ -1198,7 +1212,7 @@ SNMPCrawler::~SNMPCrawler()
 {
 }
 
-void SNMPCrawler::init(const IPAddress &_seed, 
+void SNMPCrawler::init(const InetAddr &_seed, 
 		       const string &_community,
 		       bool _recursive,
                        bool _skip_virtual,
@@ -1209,7 +1223,7 @@ void SNMPCrawler::init(const IPAddress &_seed,
 		       long _snmp_timeout,
 		       int  _dns_retries,
 		       int  _dns_timeout,
-		       const vector<IPNetwork> *_include)
+		       const vector<InetAddrMask> *_include)
 {
     include      = _include      ;
     community    = _community    ;
@@ -1234,29 +1248,32 @@ void SNMPCrawler::init(const IPAddress &_seed,
  * This method attempts to guess what interface might be associated
  * with given route. It could be more than one.
  */
-set<Interface> SNMPCrawler::guessInterface(const IPRoute &r, const map<int, Interface> &intf) const
+list<InterfaceData> SNMPCrawler::guessInterface(
+    const IPRoute &r, const map<int, InterfaceData> &intf) const
 {
-    set<Interface> res;
-    for(map<int, Interface>::const_iterator i=intf.begin(); i!=intf.end(); ++i)
-        if((*i).second.getIPNetwork().belongs(r.getGateway()))
-            res.insert((*i).second);
-    
+    list<InterfaceData> res;
+    map<int, InterfaceData>::const_iterator i;
+    for(i=intf.begin(); i!=intf.end(); ++i)
+    {
+        if ((*i).second.addr_mask.belongs(r.getGateway()))
+            res.push_back((*i).second);
+    }
     return res;
 }
 
-bool SNMPCrawler::included(const IPAddress &a) const
+bool SNMPCrawler::included(const InetAddr &a) const
 {
     if(!include)
         return true; // no include list provided. All hosts are OK.
     
-    for(vector<IPNetwork>::const_iterator i=include->begin(); i!=include->end(); ++i) {
+    for(vector<InetAddrMask>::const_iterator i=include->begin(); i!=include->end(); ++i) {
         if((*i).belongs(a)) 
             return true;
     }
     return false;
 }
 
-bool SNMPCrawler::alreadyseen(const IPAddress &a) const
+bool SNMPCrawler::alreadyseen(const InetAddr &a) const
 {
     return found.find(a) != found.end();
 }
@@ -1264,25 +1281,31 @@ bool SNMPCrawler::alreadyseen(const IPAddress &a) const
 /**
  * loopback  :  All addresses on the net  127.0.0.0/255.0.0.0
  */
-const IPNetwork SNMPCrawler::LOOPBACK_NET(IPAddress("127.0.0.0"),Netmask("255.0.0.0"));
-const Netmask   SNMPCrawler::PTP_NETMASK("255.255.255.255");
-const IPAddress SNMPCrawler::ZERO_IP("0.0.0.0");
+const InetAddrMask SNMPCrawler::LOOPBACK_NET(InetAddr::getLoopbackAddr(),
+                                          InetAddr("255.0.0.0"));
+const InetAddr SNMPCrawler::PTP_NETMASK(InetAddr::getAllOnes());
+const InetAddr SNMPCrawler::ZERO_IP("0.0.0.0");
 
-bool SNMPCrawler::isvirtual(const IPAddress &addr, const string &pa) const 
+bool SNMPCrawler::isvirtual(const InetAddr &addr, const string &pa) const 
 {
     if(!pa.length())
         return false;
 
-    for(map<IPAddress, CrawlerFind>::const_iterator i=found.begin(); i!=found.end(); ++i)
+    for(map<InetAddr, CrawlerFind>::const_iterator i=found.begin(); i!=found.end(); ++i)
     {
         const CrawlerFind &c = (*i).second ;
         
-        for(map<int, Interface>::const_iterator j=c.interfaces.begin(); j!=c.interfaces.end(); ++j)
+        map<int, InterfaceData>::const_iterator j;
+        for(j=c.interfaces.begin(); j!=c.interfaces.end(); ++j)
         {
             try
             {
-                physAddress *paddr=(*j).second.getPhysicalAddress();
-                if(paddr!=NULL && pa == paddr->getPhysAddress() && addr != (*j).second.getIPAddress())
+                const InetAddr *intf_addr =
+                    (*j).second.addr_mask.getAddressPtr();
+
+                string paddr = (*j).second.mac_addr;
+                if (paddr!="" && pa == paddr &&
+                    intf_addr!=NULL && addr != *intf_addr)
                     return true;
             } catch(FWException &ex)
             {
@@ -1295,23 +1318,23 @@ bool SNMPCrawler::isvirtual(const IPAddress &addr, const string &pa) const
     return false;
 }
 
-bool SNMPCrawler::point2point(const IPNetwork &n, const Interface *intf) const
+bool SNMPCrawler::point2point(const InetAddrMask &n,
+                              const InterfaceData& intf) const
 {
-    return n.getNetmask()==PTP_NETMASK || point2point(intf);
+    return *(n.getNetmaskPtr())==PTP_NETMASK ||
+        point2point(intf);
 }
 
-bool SNMPCrawler::point2point(const Interface *intf) const
+bool SNMPCrawler::point2point(const InterfaceData& intf) const
 {
-    if(intf)
+    int itype = intf.snmp_type;
+    for(unsigned int i=0;i<(sizeof(PTP_INTERFACE_TYPES)/sizeof(int));i++)
     {
-        int itype=intf->getInterfaceType();
-        for(unsigned int i=0;i<(sizeof(PTP_INTERFACE_TYPES)/sizeof(int));i++)
-            if(PTP_INTERFACE_TYPES[i]==itype)
-                return true;
-            else
-                if(PTP_INTERFACE_TYPES[i]>itype)
-                    return false;
-        
+        if(PTP_INTERFACE_TYPES[i]==itype)
+            return true;
+        else
+            if(PTP_INTERFACE_TYPES[i]>itype)
+                return false;
     }
     return false;
 }
@@ -1321,53 +1344,61 @@ bool SNMPCrawler::point2point(const Interface *intf) const
  * This include hosts on loopback, special addresses
  * as "0.0.0.0".
  */
-bool SNMPCrawler::special(const IPAddress &a) const 
+bool SNMPCrawler::special(const InetAddr &a) const 
 {
     return LOOPBACK_NET.belongs(a) || ZERO_IP==a;
 }
 
-bool SNMPCrawler::special(const IPNetwork &n) const
+bool SNMPCrawler::special(const InetAddrMask &n) const
 {
-    return n==LOOPBACK_NET || n.isBroadcast() || n.isMulticast() || n.getAddress()==ZERO_IP;
+    return n==LOOPBACK_NET ||
+        n.getAddressPtr()->isBroadcast() ||
+        n.getAddressPtr()->isMulticast() ||
+        n.getAddressPtr()->isAny();
 }
 
 //TODO: multiple threads (via pool).
-void SNMPCrawler::run_impl(Logger *logger,SyncFlag *stop_program) throw(FWException)
+void SNMPCrawler::run_impl(Logger *logger,
+                           SyncFlag *stop_program) throw(FWException)
 {
+    if (snmp_tmp_db==NULL)
+        snmp_tmp_db = new FWObjectDatabase();
+
     std::ostringstream str;
     time_t now=time(NULL);
     str << "SNMPCrawler started at " << asctime(localtime(&now))
         << ". Seed host: " << (*(queue.begin())).first.toString() << "\n";
     *logger << str;
-    
-    snmp_tmp_db.destroyChildren();
+
+    snmp_tmp_db->destroyChildren();
 
     SNMP_discover_query q;
     do {
         CHECK_STOP_AND_RETURN
 
-        map<IPAddress,string>::iterator i=queue.begin();
+        map<InetAddr,string>::iterator i=queue.begin();
         if(i==queue.end())
         {
             break;
         }
-        IPAddress task           = (*i).first  ;
+        InetAddr task = (*i).first  ;
         string task_phys_address = (*i).second ;
         queue.erase(i);
         
         str << "\nProcessing " << task.toString() << "\n";
+
         *logger << str;
         
 
         // Now in task we have element to probe
         
-        q.init(string(task), // fake host make - IP in dotted notation
+        q.init(task.toString(), // fake host - IP in dotted notation
                community,
                snmp_retries,  
                snmp_timeout
         );
         
-        CrawlerFind *res=new CrawlerFind();
+        CrawlerFind *res = new CrawlerFind();
         found[task] = *res; delete res;
         found[task].found_phys_addr = task_phys_address ;
         try
@@ -1392,17 +1423,16 @@ void SNMPCrawler::run_impl(Logger *logger,SyncFlag *stop_program) throw(FWExcept
             continue;
         }
 
-        map<IPAddress, string> at=q.getArtpTable();    
+        map<InetAddr, string> at=q.getArtpTable();    
         str << "Got " << long(at.size()) << " entries\n";
         *logger << str;
         
-
         int qplus=0, rplus=0, dplus=0;
-        for(map<IPAddress, string>::iterator j=at.begin();j!=at.end();++j)
+        for(map<InetAddr, string>::iterator j=at.begin();j!=at.end();++j)
         {
             CHECK_STOP_AND_RETURN
 
-            IPAddress  c = (*j).first;
+            InetAddr  c = (*j).first;
             string    pa = (*j).second;
             if(included(c) && !alreadyseen(c) && !isvirtual(c,pa) && !special(c))
             {
@@ -1438,26 +1468,32 @@ void SNMPCrawler::run_impl(Logger *logger,SyncFlag *stop_program) throw(FWExcept
             
         }
 
-        set<IPAddress> interface_broadcasts;
+        set<InetAddr> interface_broadcasts;
 
         try
         {
             q.fetchInterfaces(logger,stop_program);
             found[task].have_snmpd = true;
 
-            map<int, Interface> intf = q.getInterfaces();
-            for(map<int, Interface>::iterator j=intf.begin(); j!=intf.end(); ++j)
+            map<int, InterfaceData> intf = q.getInterfaces();
+            map<int, InterfaceData>::iterator j;
+            for(j=intf.begin(); j!=intf.end(); ++j)
             {
                 // If interface is down it will be ignored.
-                if(!(*j).second.isUp())
+                if (!(*j).second.ostatus)
                     continue;
 
-                IPNetwork net=(*j).second.getIPNetwork();
-                interface_broadcasts.insert(net.getBroadcastAddress());
+                const InetAddr *addr = (*j).second.addr_mask.getAddressPtr();
+                const InetAddr *netm = (*j).second.addr_mask.getNetmaskPtr();
+                if (addr==NULL) continue;
 
-                if(!special(net) && included(net.getAddress()) && !point2point(net, NULL))
+                InetAddrMask net(*addr, *netm);
+                interface_broadcasts.insert(*(net.getBroadcastAddressPtr()));
+
+                if(!special(net) && included(*(net.getAddressPtr())) &&
+                   !point2point(net, (*j).second))
                 {
-                    str << "Network " << string(net) << " found.\n";
+                    str << "Network " << net.toString() << " found.\n";
                     *logger << str;
                     
                     networks.insert(net);
@@ -1502,27 +1538,23 @@ void SNMPCrawler::run_impl(Logger *logger,SyncFlag *stop_program) throw(FWExcept
 
             for(vector<IPRoute>::iterator j=routes.begin(); j!=routes.end(); ++j)
             {
-                Interface intf;
+                InterfaceData intf;
                 bool have_intf;
                 
-                const Interface *real_i=(*j).getInterface();
-                if(real_i)
-                {
-                    intf      = *real_i;
-                    have_intf = true;
-                } else
-                {
-                    have_intf = false;
-                }
+                const InterfaceData& real_i = (*j).getInterface();
+                intf      = real_i;
+                have_intf = !intf.name.empty();
                 
-                IPNetwork net((*j).getDestination(), (*j).getNetmask());
+                InetAddrMask net((*j).getDestination(),
+                                 (*j).getNetmask());
                 
                 if(!have_intf)
                 {
                     // No interface reported for this route by SNMP 
                     // by we can try to guess it by route information.
                     // Since interface is used.
-                    set<Interface> gi=guessInterface(*j, q.getInterfaces());
+                    list<InterfaceData> gi =
+                        guessInterface(*j, q.getInterfaces());
                     
                     // From all resulting interfaces we select one
                     // using following rules:
@@ -1533,7 +1565,8 @@ void SNMPCrawler::run_impl(Logger *logger,SyncFlag *stop_program) throw(FWExcept
                     // 2. From interfaces which are UP, we select first one
                     // with Point-to-Point attribute enabled.
                     //
-                    for(set<Interface>::const_iterator k=gi.begin(); k!=gi.end(); ++k)
+                    list<InterfaceData>::const_iterator k;
+                    for(k=gi.begin(); k!=gi.end(); ++k)
                     {
                         if(!have_intf)
                         {
@@ -1542,7 +1575,7 @@ void SNMPCrawler::run_impl(Logger *logger,SyncFlag *stop_program) throw(FWExcept
                             have_intf = true;
                         } else 
                         {
-                            if((*k).isUp())
+                            if ((*k).ostatus)
                             {
                                 // Candidate is up
                                 // Store it.
@@ -1553,54 +1586,54 @@ void SNMPCrawler::run_impl(Logger *logger,SyncFlag *stop_program) throw(FWExcept
                                 // Candidate is down.
                                 // We take it into account only
                                 // if everything found so far is down.
-                                if(!intf.isUp())
+                                if (!intf.ostatus)
                                     intf = *k;
                             }
                         }
-                        if(have_intf && intf.isUp() && point2point(&intf))
+                        if(have_intf && intf.ostatus && point2point(intf))
                             break;
                     }
                     if(have_intf)
-                        str << "Guessed that network " << string(net)
-                            << " is using interface " << intf.getName() << "\n";
+                        str << "Guessed that network " << net.toString()
+                            << " is using interface " << intf.name << "\n";
                     *logger << str;
                     
                 }
 
                 // If route is associated with an interface which is down,
                 // ignore it.
-                if(have_intf && !intf.isUp())
+                if(have_intf && !intf.ostatus)
                 {
-                    str << "Skipping route for network " << string(net)
+                    str << "Skipping route for network " << net.toString()
                         << " which is associated with interface which is currently down.\n";
                     *logger << str;
                     
                     continue; 
                 }
 
-                if(!special(net) && included(net.getAddress()) )
+                if(!special(net) && included(*(net.getAddressPtr())) )
                 {
-                    if(point2point(net, NULL))
+                    if(point2point(net, intf))
                     {
-                        IPAddress c=net.getAddress();
+                        const InetAddr *c = net.getAddressPtr();
 
                         // For all addresses found in the routing table
                         // we must check if they are broadcast addresses
                         // for some of our interfaces, and if yes, ignore them.
                         // (see task #36520).
 
-                        if(included(c) && !alreadyseen(c) && 
-                           !isvirtual(c,"") && !special(c) && 
-                           !interface_broadcasts.count(c))
+                        if(included(*c) && !alreadyseen(*c) && 
+                           !isvirtual(*c,"") && !special(*c) && 
+                           !interface_broadcasts.count(*c))
                         {
                             if(recursive && follow_ptp)
                             {
                                 qplus++;
-                                queue[c]="";
+                                queue[*c]="";
                             } else
                             {
                                 rplus++;
-                                found[c]=CrawlerFind();
+                                found[*c]=CrawlerFind();
                             }
                         } else
                         {
@@ -1608,7 +1641,7 @@ void SNMPCrawler::run_impl(Logger *logger,SyncFlag *stop_program) throw(FWExcept
                         }
                     } else
                     {
-                        str << "Network " << string(net)
+                        str << "Network " << net.toString()
                             << " found (via "
                             << string(((*j).isDirect())?"direct":"indirect") 
                             << " route).\n";
@@ -1620,10 +1653,10 @@ void SNMPCrawler::run_impl(Logger *logger,SyncFlag *stop_program) throw(FWExcept
                     }
                 }
 
-                IPAddress gw((*j).getGateway());
+                InetAddr gw((*j).getGateway());
                 if(included(gw) && !alreadyseen(gw) && !isvirtual(gw, "") && !special(gw) && !interface_broadcasts.count(gw))
                 {
-                    bool isptp=point2point(net, &intf);
+                    bool isptp=point2point(net, intf);
                     if(recursive && (!isptp || (isptp && follow_ptp)))
                     {
                         qplus++;
@@ -1638,31 +1671,24 @@ void SNMPCrawler::run_impl(Logger *logger,SyncFlag *stop_program) throw(FWExcept
 
             if (qplus)
             {
-                str << "Adding "   << qplus
-                    << " hosts to queue\n"   ;
+                str << "Adding "   << qplus << " hosts to queue\n"   ;
                 *logger << str;
-                
             }
             if (nplus)
             {
-                str << "Adding "   << nplus
-                    << " networks to results\n"   ;
+                str << "Adding "   << nplus << " networks to results\n"   ;
                 *logger << str;
-                
             }
             if (rplus)
             {
-                str << "Adding "   << rplus
-                    << " hosts to results\n" ;
+                str << "Adding "   << rplus << " hosts to results\n" ;
                 *logger << str;
-                
             }
             if (dplus)
             {
                 str << "Skipping " << dplus
                     << " hosts as duplicate, excluded or virtual\n" ;
                 *logger << str;
-                
             }
 
         } catch(const FWException &ex)
@@ -1671,13 +1697,16 @@ void SNMPCrawler::run_impl(Logger *logger,SyncFlag *stop_program) throw(FWExcept
             str << "Failed to fetch routing table from "
                 << task.toString() << "\n";
             *logger << str;
-            
+        }
+        catch (...)
+        {
+            *logger << "Unsupported exception\n";
         }
 
         // We add interfaces _after_ fetching routing table,
         // since it's updates 'ext' attribute based on rounting
         // table.
-        found[task].interfaces=q.getInterfaces();
+        found[task].interfaces = q.getInterfaces();
         
     } while(recursive);
 
@@ -1687,57 +1716,48 @@ void SNMPCrawler::run_impl(Logger *logger,SyncFlag *stop_program) throw(FWExcept
     if(do_dns)
         bacresolve_results(logger,stop_program);
     
-    snmp_tmp_db.clear(); // at this point all children have been destroyed anyway
+    snmp_tmp_db->clear(); // at this point all children have been destroyed anyway
 
     now=time(NULL);
     str << "SNMPCrawler - done at " << asctime(localtime(&now)) << "\n";
     *logger << str;
 }
 
-void SNMPCrawler::remove_virtual(Logger *logger,SyncFlag *stop_program) throw(FWException)
+void SNMPCrawler::remove_virtual(Logger *logger, SyncFlag*) throw(FWException)
 {
     *logger << "Removing virtual IPs.\n";
-    for(map<IPAddress, CrawlerFind>::iterator j=found.begin(); j!=found.end(); ++j)
+    for(map<InetAddr, CrawlerFind>::iterator j=found.begin(); j!=found.end(); ++j)
         if(isvirtual((*j).first, (*j).second.found_phys_addr))
             found.erase(j);
 }
 
-void SNMPCrawler::bacresolve_results(Logger *logger,SyncFlag *stop_program) throw(FWException)
+void SNMPCrawler::bacresolve_results(Logger *logger,
+                                     SyncFlag *) throw(FWException)
 {
     *logger << "Resolving names\n";
         
-    set<IPAddress> resolv_set;
-    for(map<IPAddress, CrawlerFind>::iterator j=found.begin(); j!=found.end(); ++j)
-        resolv_set.insert((*j).first);
-    DNS_bulkBackResolve_query backresq(resolv_set, dns_threads,
-				       dns_retries,dns_timeout );
-    resolv_set.clear();
-        
-    try
+    set<InetAddr> resolv_set;
+    for (map<InetAddr, CrawlerFind>::iterator j=found.begin();
+         j!=found.end(); ++j)
     {
-        backresq.run_impl(logger,stop_program);
-    } catch(const FWException &ex)
-    {
-        *logger << "DNS resolver reported errors.\n";
-    }
-    *logger << "Finished.\n";
+        HostEnt he = DNS::getHostByAddr((*j).first);
 
-    map<IPAddress, HostEnt> resolv_res=backresq.getResult();
-    for(map<IPAddress, HostEnt>::iterator j=resolv_res.begin(); j!=resolv_res.end(); ++j)
-    {
         found[(*j).first].dns_ok  = true;
-        found[(*j).first].name    = (*j).second.name;
-        found[(*j).first].aliases = (*j).second.aliases;
+        found[(*j).first].name    = he.name;
+        found[(*j).first].aliases = he.aliases;
+        *logger << ((*j).first).toString() << " : " << he.name << "\n";
     }
+
+    *logger << "Finished.\n";
 }
 
 
-set<IPNetwork> SNMPCrawler::getNetworks()
+set<InetAddrMask> SNMPCrawler::getNetworks()
 {
     return networks;
 }
 
-map<IPAddress, CrawlerFind> SNMPCrawler::getAllIPs()
+map<InetAddr, CrawlerFind> SNMPCrawler::getAllIPs()
 {
     return found;
 }
