@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: Compiler.cpp 176 2008-08-19 04:29:54Z vadim $
+  $Id: Compiler.cpp 216 2008-10-25 17:19:39Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -430,14 +430,23 @@ void Compiler::_expand_addr_recursive(Rule *rule, FWObject *s,
         {
             if (Interface::cast(*i2)!=NULL)
             {
-                Interface *interface_=Interface::cast(*i2);
+                Interface *interface_ = Interface::cast(*i2);
 /*
  * Special case is loopback interface - skip it, but only if this rule is
  * not attached to loopback!
+ *
+ * Correction 10/20/2008: if user put loopback interface object into
+ * rule element, keep it. However if we expanded it from a host or
+ * firewall object, then skip it unless the rule is attached to
+ * loopback interface.
  */
-                if ( ! on_loopback && interface_->isLoopback() ) continue;
-
-                _expandInterface(interface_,ol);
+                if (interface_->isLoopback())
+                {
+                    if (RuleElement::cast(s) || on_loopback)
+                        _expandInterface(interface_,ol);
+                } else
+// this is not a loopback interface                
+                    _expandInterface(interface_,ol);
 
                 continue;
             }
@@ -1017,8 +1026,8 @@ bool Compiler::splitIfRuleElementMatchesFW::processNext()
 {
     Rule *rule=prev_processor->getNextRule(); if (rule==NULL) return false;
 
-    RuleElement    *re=RuleElement::cast(rule->getFirstByType(re_type));
-    int nre=re->size();
+    RuleElement *re = RuleElement::cast(rule->getFirstByType(re_type));
+    int nre = re->size();
 
     list<FWObject*> cl;
 
@@ -1040,7 +1049,7 @@ bool Compiler::splitIfRuleElementMatchesFW::processNext()
 	    Rule  *new_rule= Rule::cast(compiler->dbcopy->create(rule->getTypeName()) );
 	    compiler->temp_ruleset->add(new_rule);
 	    new_rule->duplicate(rule);
-            RuleElement *new_re=RuleElement::cast(new_rule->getFirstByType(re_type));
+            RuleElement *new_re = RuleElement::cast(new_rule->getFirstByType(re_type));
 	    new_re->clearChildren();
 	    new_re->setAnyElement();
 	    new_re->addRef( a );
