@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: OSConfigurator.cpp,v 1.3 2006/03/07 07:28:28 vkurland Exp $
+  $Id: OSConfigurator.cpp 123 2008-07-05 17:38:52Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -32,8 +32,9 @@ using namespace std;
 
 OSConfigurator::~OSConfigurator() {}
 
-OSConfigurator::OSConfigurator(FWObjectDatabase *_db, const string &fwname) :
-    Compiler(_db,fwname)
+OSConfigurator::OSConfigurator(FWObjectDatabase *_db,
+                               const string &fwname, bool ipv6_policy) :
+    Compiler(_db, fwname, ipv6_policy)
 {
     num_virtual_addresses_for_nat = 0;
 };
