@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler.cpp 123 2008-07-05 17:38:52Z vadim $
+  $Id: PolicyCompiler.cpp 218 2008-11-01 19:42:59Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -65,7 +65,7 @@ int PolicyCompiler::prolog()
 {
     Compiler::prolog();
 
-    FWObject  *policy=fw->getFirstByType(Policy::TYPENAME);
+    FWObject *policy = fw->getFirstByType(Policy::TYPENAME);
     assert(policy);
 
     combined_ruleset = new Policy();   // combined ruleset (all interface policies and global policy)
@@ -83,6 +83,9 @@ int PolicyCompiler::prolog()
         ruleset = policy;
     }
 
+    combined_ruleset->setName(ruleset->getName());
+    temp_ruleset->setName(ruleset->getName());
+
     string label_prefix = "";
     if (ruleset->getName() != "Policy") label_prefix = ruleset->getName();
 
@@ -126,18 +129,42 @@ int PolicyCompiler::prolog()
 /*
  * detects if tule r2 shades rule r1.
  */
-bool PolicyCompiler::checkForShadowing(PolicyRule &r1,
-                                       PolicyRule &r2)
+bool PolicyCompiler::checkForShadowing(PolicyRule &r1, PolicyRule &r2)
 {
-    if (r1.getSrc()->getNeg()) return false;
-    if (r1.getDst()->getNeg()) return false;
-    if (r1.getSrv()->getNeg()) return false;
-    if (r2.getSrc()->getNeg()) return false;
-    if (r2.getDst()->getNeg()) return false;
-    if (r2.getSrv()->getNeg()) return false;
+    RuleElement *srcrel1;
+    RuleElement *dstrel1;
+    RuleElement *srvrel1;
 
-    if (r1.getAction()==PolicyRule::Accounting  ||
-        r2.getAction()==PolicyRule::Accounting ) return false;
+    RuleElement *srcrel2;
+    RuleElement *dstrel2;
+    RuleElement *srvrel2;
+
+    FWObject::iterator i1 = r1.begin();
+    srcrel1 = RuleElement::cast(*i1);
+    i1++;
+    dstrel1 = RuleElement::cast(*i1);
+    i1++;
+    srvrel1 = RuleElement::cast(*i1);
+
+    i1 = r2.begin();
+    srcrel2 = RuleElement::cast(*i1);
+    i1++;
+    dstrel2 = RuleElement::cast(*i1);
+    i1++;
+    srvrel2 = RuleElement::cast(*i1);
+
+    if (srcrel1->getNeg()) return false;
+    if (dstrel1->getNeg()) return false;
+    if (srvrel1->getNeg()) return false;
+    if (srcrel2->getNeg()) return false;
+    if (dstrel2->getNeg()) return false;
+    if (srvrel2->getNeg()) return false;
+
+    PolicyRule::Action r1_action = r1.getAction();
+    PolicyRule::Action r2_action = r2.getAction();
+
+    if (r1_action==PolicyRule::Accounting  ||
+        r2_action==PolicyRule::Accounting ) return false;
 
     /*
      * this is delicate case: negation. We consider r2 to "shade" r1
@@ -146,8 +173,8 @@ bool PolicyCompiler::checkForShadowing(PolicyRule &r1,
      * fwb_ipt, then some of the produced rules have action Return. If
      * r2 has action != Return and r1 has action Return, we ignore r1.
      */
-    if (r1.getAction()==PolicyRule::Return  ||
-        r2.getAction()==PolicyRule::Return ) return false;
+    if (r1_action==PolicyRule::Return  ||
+        r2_action==PolicyRule::Return ) return false;
 
     /*
      * TODO: actually, route rule may shadow other rules if it
@@ -155,8 +182,8 @@ bool PolicyCompiler::checkForShadowing(PolicyRule &r1,
      * may or may not be so, depending on the platform and combination
      * of rule options.
      */
-    if (r1.getAction()==PolicyRule::Route  ||
-        r2.getAction()==PolicyRule::Route ) return false;
+    if (r1_action==PolicyRule::Route  ||
+        r2_action==PolicyRule::Route ) return false;
 
     /*
      * the problem with branching rules is that it is combination of
@@ -165,75 +192,89 @@ bool PolicyCompiler::checkForShadowing(PolicyRule &r1,
      * shadowing detection does not support this so all we can do is
      * skip rules with action Branch.
      */
-    if (r1.getAction()==PolicyRule::Branch  ||
-        r2.getAction()==PolicyRule::Branch ) return false;
+    if (r1_action==PolicyRule::Branch  ||
+        r2_action==PolicyRule::Branch ) return false;
 
+    Address  *src1;
+    Address  *dst1;
+    Service  *srv1;
 
-    //Address  *src1=getFirstSrc(&r1);
-    //Address  *dst1=getFirstDst(&r1);
-    //Service  *srv1=getFirstSrv(&r1);
+    Address  *src2;
+    Address  *dst2;
+    Service  *srv2;
 
-    //Address  *src2=getFirstSrc(&r2);
-    //Address  *dst2=getFirstDst(&r2);
-    //Service  *srv2=getFirstSrv(&r2);
-
-    Address  *src1 = static_cast<Address*>(
-        static_cast<FWReference*>(r1.getFirstByType(RuleElementSrc::TYPENAME)->front())->getPointer());
-    Address  *dst1 = static_cast<Address*>(
-        static_cast<FWReference*>(r1.getFirstByType(RuleElementDst::TYPENAME)->front())->getPointer());
-    Service  *srv1 = static_cast<Service*>(
-        static_cast<FWReference*>(r1.getFirstByType(RuleElementSrv::TYPENAME)->front())->getPointer());
-
-    Address  *src2 = static_cast<Address*>(
-        static_cast<FWReference*>(r2.getFirstByType(RuleElementSrc::TYPENAME)->front())->getPointer());
-    Address  *dst2 = static_cast<Address*>(
-        static_cast<FWReference*>(r2.getFirstByType(RuleElementDst::TYPENAME)->front())->getPointer());
-    Service  *srv2 = static_cast<Service*>(
-        static_cast<FWReference*>(r2.getFirstByType(RuleElementSrv::TYPENAME)->front())->getPointer());
+    map<int, threeTuple*>::iterator it = rule_elements_cache.find(r1.getId());
+    if (it!=rule_elements_cache.end())
+    {
+        threeTuple *tt = it->second;
+        src1 = tt->src;
+        dst1 = tt->dst;
+        srv1 = tt->srv;
+    } else
+    {
+        src1 = Address::cast(FWReference::cast(srcrel1->front())->getPointer());
+        dst1 = Address::cast(FWReference::cast(dstrel1->front())->getPointer());
+        srv1 = Service::cast(FWReference::cast(srvrel1->front())->getPointer());
+        threeTuple *tt = new struct threeTuple;
+        tt->src = src1;
+        tt->dst = dst1;
+        tt->srv = srv1;
+        rule_elements_cache[r1.getId()] = tt;
+    }
+    
+    it = rule_elements_cache.find(r2.getId());
+    if (it!=rule_elements_cache.end())
+    {
+        threeTuple *tt = it->second;
+        src2 = tt->src;
+        dst2 = tt->dst;
+        srv2 = tt->srv;
+    } else
+    {
+        src2 = Address::cast(FWReference::cast(srcrel2->front())->getPointer());
+        dst2 = Address::cast(FWReference::cast(dstrel2->front())->getPointer());
+        srv2 = Service::cast(FWReference::cast(srvrel2->front())->getPointer());
+        threeTuple *tt = new struct threeTuple;
+        tt->src = src2;
+        tt->dst = dst2;
+        tt->srv = srv2;
+        rule_elements_cache[r2.getId()] = tt;
+    }
 
+    if (src1==NULL || dst1==NULL || srv1==NULL)
+        throw FWException("Can not compare rules because rule " + 
+                          r1.getLabel()
+                          + " has a group in one of its elements. Aborting.");
 
+    if (src2==NULL || dst2==NULL || srv2==NULL)
+        throw FWException("Can not compare rules because rule " + 
+                          r2.getLabel() +
+                          " has a group in one of its elements. Aborting.");
 
     if (MultiAddressRunTime::isA(src1) || MultiAddressRunTime::isA(dst1) ||
         MultiAddressRunTime::isA(src2) || MultiAddressRunTime::isA(dst2))
         return false;
 
-    if (src1==NULL || dst1==NULL || srv1==NULL)
-        throw FWException("Can not compare rules because rule "+r1.getLabel()+" has a group in one of its elements. Aborting.");
+    PolicyRule::Direction dir1 = r1.getDirection();
+    PolicyRule::Direction dir2 = r2.getDirection();
 
-    if (src2==NULL || dst2==NULL || srv2==NULL)
-        throw FWException("Can not compare rules because rule "+r2.getLabel()+" has a group in one of its elements. Aborting.");
+    if (dir1 == PolicyRule::Both) dir1 = dir2;
+    if (dir2 == PolicyRule::Both) dir2 = dir1;
 
-//    if (! fwcompiler::intersect( r1, r2 )) return -2;
+    if (dir1 != dir2)     return false;
 
-    PolicyRule::Direction dir1=r1.getDirection();
-    PolicyRule::Direction dir2=r2.getDirection();
-
-    if (dir1==PolicyRule::Both) dir1=dir2;
-    if (dir2==PolicyRule::Both) dir2=dir1;
-
-    if (dir1!=dir2)     return false;
-
-/*  reminder: here rule2 is above rule1, so we should return true if
- *  src2 shades src1 etc.  however, if src2 happened to be a child of
- *  src1, then this is not shadowing and we return false. this is
- *  important because our definition of shadowing includes the case when
- *  addresses are equal.
- */
-//    if (srv1->getTypeName()==srv2->getTypeName() || (!srv1->isAny() && srv2->isAny()))
-//    {
     return ( 
-        fwcompiler::checkForShadowing(*src1, *src2) && 
-        fwcompiler::checkForShadowing(*dst1, *dst2) && 
-        fwcompiler::checkForShadowing(*srv1, *srv2) 
+        Compiler::checkForShadowing(*src1, *src2) && 
+        Compiler::checkForShadowing(*dst1, *dst2) && 
+        Compiler::checkForShadowing(*srv1, *srv2) 
     );
 
-//    }
+// complete: 3'5"
 
     return false;
 }
 
-bool PolicyCompiler::cmpRules(PolicyRule &r1,
-                              PolicyRule &r2)
+bool PolicyCompiler::cmpRules(PolicyRule &r1, PolicyRule &r2)
 {
     if (r1.getSrc()->getNeg()!=r2.getSrc()->getNeg()) return false;
     if (r1.getDst()->getNeg()!=r2.getDst()->getNeg()) return false;
@@ -242,7 +283,6 @@ bool PolicyCompiler::cmpRules(PolicyRule &r1,
     if (r2.getDst()->getNeg()!=r2.getDst()->getNeg()) return false;
     if (r2.getSrv()->getNeg()!=r2.getSrv()->getNeg()) return false;
 
-
     Address  *src1=getFirstSrc(&r1);
     Address  *dst1=getFirstDst(&r1);
     Service  *srv1=getFirstSrv(&r1);
@@ -251,12 +291,15 @@ bool PolicyCompiler::cmpRules(PolicyRule &r1,
     Address  *dst2=getFirstDst(&r2);
     Service  *srv2=getFirstSrv(&r2);
 
-
     if (src1==NULL || dst1==NULL || srv1==NULL)
-        throw FWException("Can not compare rules because rule "+r1.getLabel()+" has a group in one of its elements. Aborting.");
+        throw FWException("Can not compare rules because rule " +
+                          r1.getLabel() +
+                          " has a group in one of its elements. Aborting.");
 
     if (src2==NULL || dst2==NULL || srv2==NULL)
-        throw FWException("Can not compare rules because rule "+r2.getLabel()+" has a group in one of its elements. Aborting.");
+        throw FWException("Can not compare rules because rule " +
+                          r2.getLabel() +
+                          " has a group in one of its elements. Aborting.");
 
     PolicyRule::Direction dir1=r1.getDirection();
     PolicyRule::Direction dir2=r2.getDirection();
@@ -276,12 +319,14 @@ bool PolicyCompiler::ItfNegation::processNext()
 
     list<FWObject*> allInterfaces=compiler->fw->getByType(Interface::TYPENAME);
 
-    RuleElementItf *itfre=rule->getItf();
+    RuleElementItf *itfre = rule->getItf();
     if (itfre==NULL)
+    {
         compiler->abort("Missing interface RE in rule '" +
                         rule->getLabel() +
                         "' id=" + FWObjectDatabase::getStringId(rule->getId())
         );
+    }
 
     if (itfre->getNeg())
     {
@@ -526,7 +571,7 @@ bool PolicyCompiler::splitServices::processNext()
         Service *s=Service::cast( o );
         assert(s);
 
-        int proto=s->getProtocolNumber();
+        int proto = s->getProtocolNumber();
         services[proto].push_back(s);
     }
 
@@ -673,7 +718,8 @@ Address* PolicyCompiler::checkForZeroAddr::findZeroAddress(RuleElement *re)
     return a;
 }
 
-Address* PolicyCompiler::checkForZeroAddr::findHostWithNoInterfaces(RuleElement *re)
+Address* PolicyCompiler::checkForZeroAddr::findHostWithNoInterfaces(
+    RuleElement *re)
 {
     Address *a=NULL;
 
@@ -682,8 +728,9 @@ Address* PolicyCompiler::checkForZeroAddr::findHostWithNoInterfaces(RuleElement
 	FWObject *o= *i;
 	if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 	assert(o!=NULL);
-        Host      *addr=Host::cast(o);
-        if (addr!=NULL && addr->getFirstByType(Interface::TYPENAME)==NULL)
+        Host *addr = Host::cast(o);
+        // if host has child of type Interface, it must be first of the children
+        if (addr!=NULL && addr->front()!=NULL && Interface::isA(addr->front()))
         {
             a=addr;
             break;
