@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
    
-  $Id: ipAddressTest.cpp,v 1.3 2006/03/05 06:07:10 vkurland Exp $
+  $Id: ipAddressTest.cpp 918 2006-03-05 06:07:10Z vkurland $
 
 
   This program is free software which we release under the GNU General Public
@@ -115,9 +115,9 @@ void IPAddressTest(FWObjectDatabase *objdb)
          << " (" << size << " addresses) "
          << " to networks" << endl;
 
-    vector<IPNetwork> vn=libfwbuilder::convertAddressRange(*a1,*a2);
+    vector<InetAddrMask> vn=libfwbuilder::convertAddressRange(*a1,*a2);
 
-    for (vector<IPNetwork>::iterator i=vn.begin(); i!=vn.end(); i++)
+    for (vector<InetAddrMask>::iterator i=vn.begin(); i!=vn.end(); i++)
     {
         IPAddress bcast=i->getBroadcastAddress();
 
